@@ -1,5 +1,5 @@
 localmoran_perm <- function(x, listw, nsim=499L, zero.policy=NULL,
-    na.action=na.fail, alternative = "two.sided", p.adjust.method="none",
+    na.action=na.fail, alternative = "two.sided",
     mlvar=TRUE, spChk=NULL, adjust.x=FALSE, sample_Ei=TRUE, iseed=NULL) {
     alternative <- match.arg(alternative, c("greater", "less", "two.sided"))
     stopifnot(is.vector(x))
@@ -103,30 +103,21 @@ localmoran_perm <- function(x, listw, nsim=499L, zero.policy=NULL,
     }
 
     crd <- card(listw$neighbours)
-    permI_int <- function(i, zi, z_i, crdi, wtsi, nsim, Ii, alternative,
-        sample_Ei, EIci, probs) {
+    permI_int <- function(i, zi, z_i, crdi, wtsi, nsim, Ii) {
         res_i <- rep(as.numeric(NA), 8)       
         if (crdi > 0) {
             sz_i <- matrix(sample(z_i, size=crdi*nsim, replace=TRUE),
                 ncol=crdi, nrow=nsim)
             lz_i <- sz_i %*% wtsi
             res_p <- (zi/s2)*lz_i
-            if (sample_Ei) res_i[1] <- mean(res_p)
-            else res_i[1] <- EIci
+            res_i[1] <- mean(res_p)
             res_i[2] <- var(res_p)
-            res_i[3] <- (Ii - res_i[1])/sqrt(res_i[2])
-            if (alternative == "two.sided") 
-                res_i[4] <- 2 * pnorm(abs(res_i[3]), lower.tail=FALSE)
-            else if (alternative == "greater") 
-                res_i[4] <- pnorm(res_i[3], lower.tail=FALSE)
-            else res_i[4] <- pnorm(res_i[3])
             xrank <- rank(c(res_p, Ii))[(nsim + 1L)]
-	    res_i[5] <- probs[xrank]
+	    res_i[5] <- xrank
             rnk0 <- as.integer(sum(res_p >= Ii))
-# 210811 from https://github.com/pysal/esda/blob/4a63e0b5df1e754b17b5f1205b8cadcbecc5e061/esda/crand.py#L211-L213
             drnk0 <- nsim - rnk0
             rnk <- ifelse(drnk0 < rnk0, drnk0, rnk0)
-            res_i[6] <- (rnk + 1.0) / (nsim + 1.0)
+            res_i[6] <- rnk0
             res_i[7] <- e1071::skewness(res_p)
             res_i[8] <- e1071::kurtosis(res_p)
         }
@@ -135,7 +126,6 @@ localmoran_perm <- function(x, listw, nsim=499L, zero.policy=NULL,
 
     lww <- listw$weights
     Iis <- res[,1]
-
     if (parallel == "snow") {
       if (requireNamespace("parallel", quietly = TRUE)) {
         sI <- parallel::splitIndices(n, length(cl))
@@ -145,16 +135,11 @@ localmoran_perm <- function(x, listw, nsim=499L, zero.policy=NULL,
         assign("lww", lww, envir=env)
         assign("nsim", nsim, envir=env)
         assign("Iis", Iis, envir=env)
-        assign("alternative", alternative, envir=env)
-        assign("EIc", EIc, envir=env)
-        assign("sample_Ei", sample_Ei, envir=env)
-        assign("probs", probs, envir=env)
-        parallel::clusterExport(cl, varlist=c("z", "crd", "lww", "nsim", "Iis",
-            "alternative", "EIc", "sample_Ei", "probs"), envir=env)
+        parallel::clusterExport(cl, varlist=c("z", "crd", "lww", "nsim",
+            "Iis"), envir=env)
         if (!is.null(iseed)) parallel::clusterSetRNGStream(cl, iseed = iseed)
         oo <- parallel::clusterApply(cl, x = sI, fun=lapply, function(i) {
- 	    permI_int(i, z[i], z[-i], crd[i], lww[[i]], nsim, Iis[i],
-            alternative, sample_Ei, EIc[i], probs)})
+ 	    permI_int(i, z[i], z[-i], crd[i], lww[[i]], nsim, Iis[i])})
         out <- do.call("rbind", do.call("c", oo))
         rm(env)
       } else {
@@ -167,8 +152,7 @@ localmoran_perm <- function(x, listw, nsim=499L, zero.policy=NULL,
         RNGkind("L'Ecuyer-CMRG")
         if (!is.null(iseed)) set.seed(iseed)
         oo <- parallel::mclapply(sI, FUN=lapply, function(i) {permI_int(i,
-            z[i], z[-i], crd[i], lww[[i]], nsim, Iis[i], alternative, 
-            sample_Ei, EIc[i], probs)}, mc.cores=ncpus)
+            z[i], z[-i], crd[i], lww[[i]], nsim, Iis[i])}, mc.cores=ncpus)
         RNGkind(oldRNG[1])
         out <- do.call("rbind", do.call("c", oo))
       } else {
@@ -177,17 +161,25 @@ localmoran_perm <- function(x, listw, nsim=499L, zero.policy=NULL,
     } else {
         if (!is.null(iseed)) set.seed(iseed)
         oo <- lapply(1:n, function(i) permI_int(i, z[i], z[-i], 
-            crd[i], lww[[i]], nsim, Iis[i], alternative, 
-            sample_Ei, EIc[i], probs))
+            crd[i], lww[[i]], nsim, Iis[i]))
         out <- do.call("rbind", oo)
     }
 
-    res[,2] <- out[,1]
+    if (sample_Ei) res[,2] <- out[,1]
+    else  res[,2] <- EIc
     res[,3] <- out[,2]
-    res[,4] <- out[,3]
-    res[,5] <- p.adjustSP(out[,4], listw$neighbours, method=p.adjust.method)
-    res[,6] <- p.adjustSP(out[,5], listw$neighbours, method=p.adjust.method)
-    res[,7] <- p.adjustSP(out[,6], listw$neighbours, method=p.adjust.method)
+    res[,4] <- (res[,1] - res[,2])/sqrt(res[,3])
+    if (alternative == "two.sided") 
+        res[,5] <- 2 * pnorm(abs(res[,4]), lower.tail=FALSE)
+    else if (alternative == "greater") 
+        res[,5] <- pnorm(res[,4], lower.tail=FALSE)
+    else res[,5] <- pnorm(res[,4])
+    res[,6] <- probs[as.integer(out[,5])]
+# 210811 from https://github.com/pysal/esda/blob/4a63e0b5df1e754b17b5f1205b8cadcbecc5e061/esda/crand.py#L211-L213
+    rnk0 <- as.integer(out[,6])
+    drnk0 <- nsim - rnk0
+    rnk <- ifelse(drnk0 < rnk0, drnk0, rnk0)
+    res[,7] <- (rnk + 1.0) / (nsim + 1.0)
     res[,8] <- out[,7]
     res[,9] <- out[,8]
     if (!is.null(na.act) && excl) {
@@ -204,7 +196,7 @@ localmoran_perm <- function(x, listw, nsim=499L, zero.policy=NULL,
 
 
 
-localG_perm <- function(x, listw, nsim=499, zero.policy=NULL, spChk=NULL, return_internals=FALSE, iseed=NULL) {
+localG_perm <- function(x, listw, nsim=499, zero.policy=NULL, spChk=NULL, return_internals=TRUE, alternative = "two.sided", iseed=NULL) {
     if (!inherits(listw, "listw"))
 	stop(paste(deparse(substitute(listw)), "is not a listw object"))
     if (!is.numeric(x))
@@ -250,17 +242,21 @@ localG_perm <- function(x, listw, nsim=499, zero.policy=NULL, spChk=NULL, return
         stopifnot(length(iseed) == 1L)
     }
 
-    permG_int <- function(i, xi, x_i, crdi, wtsi, nsim) {
+    permG_int <- function(i, xi, x_i, crdi, wtsi, nsim, Gi) {
+        res_i <- rep(as.numeric(NA), 6)
         if (crdi > 0) {
             sx_i <- matrix(sample(x_i, size=crdi*nsim, replace=TRUE),
                 ncol=crdi, nrow=nsim)
             lx_i <- sx_i %*% wtsi
             res_p <- lx_i/(x_star-xi)
-            res <- c(mean(res_p), var(res_p))
-        } else {
-            res <- c(as.numeric(NA), as.numeric(NA))
+            res_i[1] <- mean(res_p)
+            res_i[2] <- var(res_p)
+	    res_i[3] <- rank(c(res_p, Gi))[(nsim + 1L)]
+            res_i[4] <- as.integer(sum(res_p >= Gi))
+            res_i[5] <- e1071::skewness(res_p)
+            res_i[6] <- e1071::kurtosis(res_p)
         }
-        res
+        res_i
     }
 
     crd <- card(listw$neighbours)
@@ -274,11 +270,12 @@ localG_perm <- function(x, listw, nsim=499, zero.policy=NULL, spChk=NULL, return
         assign("crd", crd, envir=env)
         assign("lww", lww, envir=env)
         assign("nsim", nsim, envir=env)
-        parallel::clusterExport(cl, varlist=c("x", "crd", "lww", "nsim"),
+        assign("G", G, envir=env)
+        parallel::clusterExport(cl, varlist=c("x", "crd", "lww", "nsim", "G"),
             envir=env)
         if (!is.null(iseed)) parallel::clusterSetRNGStream(cl, iseed = iseed)
         oo <- parallel::clusterApply(cl, x = sI, fun=lapply, function(i) {
- 	    permG_int(i, x[i], x[-i], crd[i], lww[[i]], nsim)})
+ 	    permG_int(i, x[i], x[-i], crd[i], lww[[i]], nsim, G[i])})
         out <- do.call("rbind", do.call("c", oo))
         rm(env)
       } else {
@@ -291,7 +288,7 @@ localG_perm <- function(x, listw, nsim=499, zero.policy=NULL, spChk=NULL, return
         RNGkind("L'Ecuyer-CMRG")
         if (!is.null(iseed)) set.seed(iseed)
         oo <- parallel::mclapply(sI, FUN=lapply, function(i) {permG_int(i,
-            x[i], x[-i], crd[i], lww[[i]], nsim)}, mc.cores=ncpus)
+            x[i], x[-i], crd[i], lww[[i]], nsim, G[i])}, mc.cores=ncpus)
         RNGkind(oldRNG[1])
         out <- do.call("rbind", do.call("c", oo))
       } else {
@@ -300,7 +297,7 @@ localG_perm <- function(x, listw, nsim=499, zero.policy=NULL, spChk=NULL, return
     } else {
         if (!is.null(iseed)) set.seed(iseed)
         oo <- lapply(1:n, function(i) permG_int(i, x[i], x[-i], 
-            crd[i], lww[[i]], nsim))
+            crd[i], lww[[i]], nsim, G[i]))
         out <- do.call("rbind", oo)
     }
     EG <- out[,1]
@@ -309,7 +306,37 @@ localG_perm <- function(x, listw, nsim=499, zero.policy=NULL, spChk=NULL, return
     res <- (G - EG)
     res <- res / sqrt(VG)
     if (return_internals) {
-        attr(res, "internals") <- cbind(G=G, EG=EG, VG=VG)
+        gr <- punif((1:(nsim+1))/(nsim+1), 0, 1)
+        ls <- rev(gr)
+        ts <- (ifelse(gr > ls, ls, gr))*2
+        if (alternative == "two.sided") {
+            probs <- ts
+            Prname <- "Pr(z != E(Gi))"
+        } else if (alternative == "greater") {
+            Prname <- "Pr(z > E(Gi))"
+            probs <- gr
+        } else {
+            Prname <- "Pr(z < E(Gi))"
+            probs <- ls
+        }
+        Prname_rank <- paste0(Prname, " Sim")
+        Prname_sim <- "Pr(folded) Sim"
+        if (alternative == "two.sided") 
+            pv <- 2 * pnorm(abs(res), lower.tail=FALSE)
+        else if (alternative == "greater") 
+            pv <- pnorm(res, lower.tail=FALSE)
+        else pv <- pnorm(res)
+        pu <- probs[as.integer(out[,3])]
+# 210811 from https://github.com/pysal/esda/blob/4a63e0b5df1e754b17b5f1205b8cadcbecc5e061/esda/crand.py#L211-L213
+        rnk0 <- as.integer(out[,4])
+        drnk0 <- nsim - rnk0
+        rnk <- ifelse(drnk0 < rnk0, drnk0, rnk0)
+        pr <- (rnk + 1.0) / (nsim + 1.0)
+        resint <- cbind(G=G, EG=EG, VG=VG, pv=pv, pu=pu, pr=pr,
+            sk=out[,5], ku=out[,6])
+        colnames(resint) <- c("Gi", "E.Gi", "Var.Gi", Prname,
+            Prname_rank, Prname_sim, "Skewness", "Kurtosis")
+        attr(res, "internals") <- resint
     }
     attr(res, "gstari") <- gstari
     attr(res, "call") <- match.call()
