@@ -1,22 +1,6 @@
 # Copyright 2001-7 by Roger Bivand
 #
 
-as.spam.listw <- function(listw) {
-    if (requireNamespace("spam", quietly = TRUE)) {
-#if (!require(spam)) stop("spam not available")
-        N <- length(listw$neighbours)
-        W_sn <- listw2sn(listw)
-        rpts <- as.integer(cumsum(c(1, card(listw$neighbours))))
-        W <- new("spam", entries=W_sn$weights, colindices=W_sn$to,
-            rowpointers=rpts, dimension=as.integer(c(N, N)))
-        stopifnot(spam::validate_spam(W))
-        return(W)
-    } else stop("spam not available")
-}
-
-listw2U_spam <- function(lw) 0.5 * (lw + t(lw))
-
-
 listw2sn <- function(listw) {
 	if(!inherits(listw, "listw")) stop("not a listw object")
 	cardw <- card(listw$neighbours)
