@@ -1,4 +1,4 @@
-# Copyright 2001-2010 by Roger Bivand
+# Copyright 2001-2019 by Roger Bivand
 # Upgrade to sp classes February 2007
 #
 
@@ -26,26 +26,7 @@ summary.nb <- function(object, coords=NULL, longlat=NULL, scale=1, ...) {
 	    max.nb, " link", ifelse(max.nb < 2L, "", "s"), "\n", sep="")
     }
     if(!is.null(coords)) {
-   	if (inherits(coords, "SpatialPoints")) {
-      		if ((is.null(longlat) || !is.logical(longlat)) 
- 		    && !is.na(is.projected(coords)) && !is.projected(coords)) {
-         		longlat <- TRUE
-      		} else longlat <- FALSE
-      		coords <- coordinates(coords)
-   	} else if (is.null(longlat) || !is.logical(longlat)) longlat <- FALSE
-        if (!is.matrix(coords)) stop("Data not in matrix form")
-        if (any(is.na(coords))) stop("Data include NAs")
-        stopifnot(ncol(coords) == 2)
-        if (longlat) {
-            bb <- bbox(coords)
-            if (!.ll_sanity(bb))
-                warning("Coordinates are not geographical: longlat argument wrong")
-        }
-        np <- nrow(coords)
-	if(np != n.nb) stop("Number of coords not equal to number of regions")
-        dimension <- ncol(coords)
-	dlist <- .Call("nbdists", nb, as.matrix(coords), as.integer(np), 
-	    as.integer(dimension), as.integer(longlat), PACKAGE="spdep")[[1]]
+        dlist <- nbdists(nb, coords, longlat=longlat)
 	cat("Summary of link distances:\n")
 	print(summary(unlist(dlist)))
 	stem(unlist(dlist), scale=scale)
@@ -67,7 +48,8 @@ print.nb <- function(x, ...) {
     cat("Average number of links:", mean(c.nb), "\n")
     if(any(c.nb == 0)) cat(length(c.nb[c.nb == 0]), " region", 
         ifelse(length(c.nb[c.nb == 0]) < 2L, "", "s"), " with no links:\n",
-	paste(regids[which(c.nb == 0)], collapse=" "), "\n", sep="")
+	paste(strwrap(paste(regids[which(c.nb == 0)], collapse=" ")),
+        collapse="\n"), "\n", sep="")
     res <- is.symmetric.nb(nb, verbose=FALSE)
     if (!res) cat("Non-symmetric neighbours list\n")
     invisible(x)
